#include "fire_up.h"

FireUpApp::FireUpApp()
	: BApplication( APP_SIG )
{
	removeFromDeskbar(NULL);
	_directToDeskbar = false;
	
	//since the tray item shows an icon, and the class TrayView needs to be able to know the location
	//of the executing binary, we write into the settings file this critical information when the app is fired up
	app_info info;
	be_app->GetAppInfo(&info);
	
	BEntry appEntry(&info.ref);
	BPath appPath;
	
	appEntry.GetPath(&appPath);
	appPath.GetParent(&appPath);
	
	//now, put the path into the settings file
	DeskscopeSettings settings;
	settings.SetAppPath(appPath);		
		
	return;
}

FireUpApp::~FireUpApp()
{
	return;
}

void FireUpApp::ArgvReceived(int32 argc, char ** argv)
{
	BString option;
	bool inDeskbar = false, persist = false;

	for (int32 i = 1; i < argc; i++)
	{
		option = argv[i];
		if (option.IFindFirst("deskbar") != B_ERROR)
			inDeskbar = true;
			
		if (option.IFindFirst("persist") != B_ERROR)
			persist = true;
	}	
	
	if (inDeskbar && !persist)
	{
		printf("Deskcope being put into Tray (one shot)...\n");

		BDeskbar db;
		db.AddItem(new TrayView);
		_directToDeskbar = true;

	}
	else if (inDeskbar && persist)
	{
		printf("Deskcope being put into Tray (persistant)...\n");

		BDeskbar db;
		
		BRoster roster;
		entry_ref ref;
		roster.FindApp(APP_SIG, &ref);
		int32 id;
		db.AddItem(&ref, &id);
		
		_directToDeskbar = true;
	}
	else
	{
		printf("\nUsage: Deskscope [options]\n\t--deskbar\twill not open window, will just put Deskscope into tray\n\t--persist will put Descope into tray such that it remains between bootings\n");	
	}
	
	be_app_messenger.SendMessage(B_QUIT_REQUESTED);
}

void FireUpApp::ReadyToRun()
{
	if (!_directToDeskbar)
	{
		FrontWindow *tw = new FrontWindow();
		tw->Show();
		printf("\nUsage: Deskscope [options]\n\t--deskbar\twill not open window, will just put Deskscope into tray\n\t--persist will put Descope into tray such that it remains between bootings\n");	
	}
}

//***************************************************

FrontWindow::FrontWindow()
	: BWindow( BRect( 50, 50, 380, 200 ),
	"Deskscope",
	B_TITLED_WINDOW,
	B_NOT_ZOOMABLE | B_NOT_RESIZABLE | B_ASYNCHRONOUS_CONTROLS)
{
	app_info info;
	be_app->GetAppInfo(&info);
	
	BEntry appEntry(&info.ref);
	BPath appPath;
	appEntry.GetPath(&appPath);
	appPath.GetParent(&appPath);	

	//do layout of components
	ZLayoutView *top = new ZLayoutView(Bounds(), "top", new ZBorderLayout(), B_FOLLOW_ALL_SIDES);
	AddChild(top);

	//make our GUI components	
	_tv = new TrayView;
	_lv = new LogoView(appPath, Z_NULLRECT);
	BButton *adder = new BButton(Z_NULLRECT, "add_to_desk", "Add To Deskbar", new BMessage(ADD_TO_TRAY));
	BButton *settings = new BButton(Z_NULLRECT, "settings", "Settings...", new BMessage(OPEN_SETTINGS));
	BCheckBox *persist = new BCheckBox(Z_NULLRECT, "persist", "Stay in deskbar tray between bootings", new BMessage('zPER'));	
	persist->SetValue(1);
	_persist = true;

	ZLayoutView *north = new ZLayoutView(Z_NULLRECT, "north", new ZVerticalLayout());
	ZLayoutView *nRowOne = new ZLayoutView(Z_NULLRECT, "north", new ZHorizontalLayout());	
	ZLayoutView *nRowTwo = new ZLayoutView(Z_NULLRECT, "north", new ZHorizontalLayout());

	nRowOne->AddChild(settings);
	nRowOne->AddChild(adder);
	//((ZHorizontalLayout *)nRowOne->LayoutManager())->PutSpring(1);

	nRowTwo->AddChild(persist);
	nRowTwo->AddChild(_tv);
	((ZHorizontalLayout *)nRowTwo->LayoutManager())->PutSpring(1);
	
	north->AddChild(nRowOne);
	north->AddChild(new ZSeparator(Z_NULLRECT, "sep0", ZSeparator::HORIZONTAL));
	north->AddChild(nRowTwo);	

	top->AddChildAt(north, ZBorderLayout::NORTH);	
	top->AddChildAt(_lv, ZBorderLayout::CENTER);

	top->ResizeToPreferred();

	settings->SetTarget(_tv);
}

FrontWindow::~FrontWindow()
{
}

bool FrontWindow::QuitRequested()
{
	_lv->StopAnimation();

	be_app->PostMessage(B_QUIT_REQUESTED);
	return true;
}

void FrontWindow::MessageReceived(BMessage* message)
{
	switch(message->what)
	{
		case 'zPER':
		{
			long v;
			message->FindInt32("be:value", &v);
			
			_persist = (bool) v;		
			break;
		}

		case ADD_TO_TRAY:
		{
			BDeskbar db;

			if (!_persist)
			{
				printf("Deskcope being put into Tray (one shot)...\n");	

				db.AddItem(new TrayView);
			}
			else
			{
				printf("Deskcope being put into Tray (persistant)...\n");
		
				BRoster roster;
				entry_ref ref;
				roster.FindApp(APP_SIG, &ref);
				int32 id;
				db.AddItem(&ref, &id);
			}
			
			break;
		}
		default:
		{
			BWindow::MessageReceived(message);
			break;
		}
	}
}

void FrontWindow::Hide()
{
	BWindow::Hide();
	_lv->StopAnimation();

}

void FrontWindow::Show()
{
	BWindow::Show();
}

void FrontWindow::WindowActivated(bool a)
{
	if (a)
	{
		_lv->StartAnimation();	
	}
	else
	{
		_lv->StopAnimation();	
	}

}
