#include "logoview.h"

LogoView::LogoView(BPath appPath, BRect frame)
	:BView(frame, "logoview", B_FOLLOW_LEFT | B_FOLLOW_TOP, B_WILL_DRAW)
{
	_swatch.Set(0,0, 60, 60);
	
	BString logoPath(appPath.Path());
	logoPath << "/icons/deskscope8.png";
		
	_logo = BTranslationUtils::GetBitmapFile(logoPath.String());

	_dx = 1; 
	_dy = 1;
	_updater = NULL;
	
	StopAnimation();
}

LogoView::~LogoView()
{
	if (_logo) delete _logo;
	if (_updater) delete _updater;
}

void LogoView::GetPreferredSize(float *width, float *height)
{
	if (_logo)
	{
		BRect f(_logo->Bounds());
		*width = f.Width();
		*height = f.Height();
	}
	else
	{
		*width = Bounds().Width();
		*height = Bounds().Height();
	}
}

void LogoView::StartAnimation(uint32 fps = 24)
{
	if (_updater) return;
	_frame = 0;

	//BMessenger uses delays in millionths of a second
	uint32 oneMillion = 1000000;
	uint32 delay = oneMillion / fps;
	
	//to create ghosting effect
	SetDrawingMode(B_OP_BLEND);
	
	_updater = new BMessageRunner(this, new BMessage(Z_REDRAW_LOGO), delay, -1);
}

void LogoView::StopAnimation()
{
	if (_updater) delete _updater;
	_updater = NULL;
	_frame = 0;
	//so we can draw the still image
	SetDrawingMode(B_OP_ADD);		
}

void LogoView::Draw(BRect updateRect )
{
	float speed = 5;
	float magnification = 10;
	BRect b = Bounds();


	if (!_updater)
	{
		SetDrawingMode(B_OP_COPY);
		if (_logo) DrawBitmap(_logo);
	}
	else
	{	
		SetDrawingMode(B_OP_BLEND);
		
		_swatch.OffsetBy(_dx , _dy);


		float change = sin((double)_frame * 3.14159/180.0);
		if (change < 0) change = -change;
		
		if (_swatch.right > b.right) 
		{
			_dx = - (speed * change);
			if (_dx > -1) _dx = -1;
		}
		
		if (_swatch.bottom > b.bottom)
		{
			_dy = - (change * speed);
			if (_dy > -1) _dy = -1;
		}
		
		if (_swatch.left < 0)
		{
			 _dx = speed * change;
			 if (_dx < 1) _dx = 1;
		}
		
		if (_swatch.top < 0)
		{
			_dy = speed * change;
			if (_dy < 1) _dy = 1;
		}
		
		BRect mag = _swatch.InsetByCopy(BPoint(magnification * change, magnification * change));
		
		if (_frame == 0)
		{
			drawing_mode dm = DrawingMode();
			SetDrawingMode(B_OP_COPY);
			SetHighColor(ViewColor());
			FillRect(b);
			
			SetDrawingMode(dm);
		}
		
		if (_logo)
		{
			DrawBitmap(_logo);
			BRect logoBounds = _logo->Bounds();
			
			if (b.right > logoBounds.right)
			{
				BRect chunk(logoBounds.right + 1, 0, b.right, b.bottom);
				drawing_mode dm = DrawingMode();
				SetDrawingMode(B_OP_COPY);
				SetHighColor(ViewColor());
				FillRect(chunk);
				SetDrawingMode(dm);
			}
			
			if (b.bottom > logoBounds.bottom)
			{
				BRect chunk(0, logoBounds.bottom + 1, b.right, b.bottom);
				drawing_mode dm = DrawingMode();
				SetDrawingMode(B_OP_COPY);
				SetHighColor(ViewColor());
				FillRect(chunk);
				SetDrawingMode(dm);
			}
			
			DrawBitmap(_logo, mag, _swatch);
		}
		
	}
	_frame ++;
}

void LogoView::AttachedToWindow()
{		
	SetViewColor(B_TRANSPARENT_COLOR);
	StopAnimation();	
	ResizeToPreferred();
}

void LogoView::MessageReceived(BMessage* message)
{
	switch(message->what)
	{
		case Z_REDRAW_LOGO:
		{
			Invalidate();
			break;
		}
	
		default:
			BView::MessageReceived(message);	
	}
}
