#include "tray_view.h"

extern "C" _EXPORT BView *instantiate_deskbar_item(void)
{		
	return (new TrayView);
}


long removeFromDeskbar(void *)
{	
	BDeskbar db;
	if (db.RemoveItem("Deskscope") != B_OK)
		printf("Unable to remove Deskscope from BDeskbar\n");
		
	return 0;
}

//**************************************************

ConfigMenu::ConfigMenu(TrayView *tv, bool useMag)
	:BPopUpMenu("config_popup", false, false){

	BMenuItem *ed = new BMenuItem("Use Eyedropper", new BMessage(USE_EYEDROPPER));
	BMenuItem *mg = new BMenuItem("Use Mag", new BMessage(USE_MAGNIFIER));
	
	mg->SetMarked(useMag);
	ed->SetMarked(!useMag);
	
	AddItem(ed);
	AddItem(mg);
	AddSeparatorItem();
	AddItem(new BMenuItem("Settings...", new BMessage(OPEN_SETTINGS)));
	AddItem(new BMenuItem("Remove from tray", new BMessage(REMOVE_FROM_TRAY)));
		
	SetTargetForItems(tv);
	SetAsyncAutoDestruct(true);
}

ConfigMenu::~ConfigMenu() {}

//************************************************

TrayView::TrayView()
	:BView(BRect(0, 0, B_MINI_ICON, B_MINI_ICON -1), "Deskscope", B_FOLLOW_LEFT | B_FOLLOW_TOP, B_WILL_DRAW){
	_init(); 	//Initialization common to both constructors
}

//Rehydratable constructor
TrayView::TrayView(BMessage *mdArchive):BView(mdArchive){
	_init();		//As above
}

void TrayView::GetPreferredSize(float *w, float *h)
{
	*w = B_MINI_ICON;
	*h = B_MINI_ICON - 1;
}

void TrayView::_init()
{
	DeskscopeSettings settings;
	_appPath = settings.AppPath();

	
	_magIcon = NULL;
	_eyedropIcon = NULL;

	_magWindow = new MagnifyWindow();
	_eyedropWindow = new SwatchWindow();

	//start their message loops
	_magWindow->Run();
	_eyedropWindow->Run();
	
	_magMsngr = new BMessenger(NULL, _magWindow);
	_eyedropMsngr = new BMessenger(NULL, _eyedropWindow);

	//hide the buggers
	_magMsngr->SendMessage(Z_GO_AWAY);
	_eyedropMsngr->SendMessage(Z_GO_AWAY);
	
	_settingsWindow = new SettingsWindow(_eyedropMsngr, _magMsngr);
	_settingsWindow->Run(); //start message loop
	if (_settingsWindow->Lock())
	{
		if (!_settingsWindow->IsHidden())_settingsWindow->Hide(); //hide it
		_settingsWindow->Unlock();
	}
	
	_useMag = true;
	_running = false;

	//dtermine paths to icon files based on app path in settings file	
	BString eyedropIconPath(_appPath.Path());
	eyedropIconPath << "/icons/icon_eyedropper.png";

	BString magIconPath(_appPath.Path());
	magIconPath << "/icons/icon_magnifier.png";
			
	_eyedropIcon = BTranslationUtils::GetBitmapFile(eyedropIconPath.String());
	_magIcon = BTranslationUtils::GetBitmapFile(magIconPath.String());
	
	SetDrawingMode(B_OP_ALPHA);
	SetFlags(Flags() | B_WILL_DRAW);
}

TrayView::~TrayView(){
	if (_magWindow->Lock()) _magWindow->Quit();
	if (_eyedropWindow->Lock()) _eyedropWindow->Quit();
	if (_settingsWindow->Lock()) _settingsWindow->Quit();
	
	if (_magIcon) delete _magIcon;
	if (_eyedropIcon) delete _eyedropIcon;

	delete _magMsngr;
	delete _eyedropMsngr;

	return;
}

//Dehydrate into a message (called by the DeskBar)
status_t TrayView::Archive(BMessage *data, bool deep = true) const {

	status_t error=BView::Archive(data, deep);	
	data->AddString("add_on", APP_SIG);
	data->AddFlat("_appPath", (BFlattenable *) &_appPath);

	return B_NO_ERROR;
}

//Rehydrate the View from a given message (called by the DeskBar)
TrayView *TrayView::Instantiate(BMessage *data) {

	if (!validate_instantiation(data, "TrayView"))
	{
		return NULL;
	}
	
	return (new TrayView(data));
}

void TrayView::AttachedToWindow() {
	if(Parent())
		SetViewColor(Parent()->ViewColor());

	_magWindow->SetOwner(this);
	_eyedropWindow->SetOwner(this);
}

void TrayView::Draw(BRect updaterect) {
	BRect bnds(Bounds());
	
	if (Parent()) SetHighColor(Parent()->ViewColor());
	else SetHighColor(189, 186, 189, 255);
	FillRect(bnds);
	
	if (_useMag)
	{
		if (_magIcon) DrawBitmap(_magIcon);
	}
	else
	{
		if (_eyedropIcon) DrawBitmap(_eyedropIcon);
	}



	BRect r(bnds);
	r.InsetBy(1,1);
	r.top = r.bottom - 4;
	r.left = r.right - 4;
	SetHighColor(255, 255, 255, 255);
	
	BPoint n, s, e, w;
	n.x = (r.left + r.right) / 2;
	n.y = r.top;
	
	s.x = n.x; s.y = r.bottom;
	
	w.x = r.left; w.y = (r.bottom + r.top)/2;
	e.x = r.right; e.y = w.y;
	
	if (!_running) StrokeLine(n, s);
	StrokeLine(w, e);	
}

void TrayView::MouseDown(BPoint where) {
	BWindow *window = Window();	/*To handle the MouseDown message*/
	if (!window)	/*Check for proper instantiation*/
		return;
		
	BMessage *mouseMsg = window->CurrentMessage();
	if (!mouseMsg)	/*Check for existence*/
		return;
		
	if (mouseMsg->what == B_MOUSE_DOWN) {
		/*Variables for storing the button pressed / modifying key*/
		uint32 	buttons = 0;
		uint32  modifiers = 0;
		
		/*Get the button pressed*/
		mouseMsg->FindInt32("buttons", (int32 *) &buttons);
		/*Get modifier key (if any)*/
		mouseMsg->FindInt32("modifiers", (int32 *) &modifiers);
		
		/*Now perform action*/
		switch(buttons) {
			case B_PRIMARY_MOUSE_BUTTON:
			{
				if (_useMag)
				{
					if (_magWindow->Lock())
					{
						if (_magWindow->IsHidden()){
							_magMsngr->SendMessage(Z_WAKE_UP);
							_running = true;
						}
						else{
							_magMsngr->SendMessage(Z_GO_AWAY);
							_running = false;
						}
						_magWindow->Unlock();
					}
				}
				else
				{
					if (_eyedropWindow->Lock())
					{
						if (_eyedropWindow->IsHidden()){
							_eyedropMsngr->SendMessage(Z_WAKE_UP);
							_running = true;
						}
						else{
							_eyedropMsngr->SendMessage(Z_GO_AWAY);
							_running = false;
						}
						_eyedropWindow->Unlock();
					}
				}
				Invalidate();
				break;
			}
			case B_SECONDARY_MOUSE_BUTTON: 
			{
				ConvertToScreen(&where);

				//menu will delete itself (see constructor of ConfigMenu),
				//so all we're concerned about is calling Go() asynchronously
				ConfigMenu *menu = new ConfigMenu(this, _useMag);
				menu->Go(where, true, true, ConvertToScreen(Bounds()), true); 

				break;
			}
		}
	}
}	

void TrayView::MessageReceived(BMessage* message)
{
	switch(message->what)
	{		
		case USE_EYEDROPPER:
		{
			if (_magWindow->Lock()){
				_magMsngr->SendMessage(Z_GO_AWAY);
				_running = false;
				_magWindow->Unlock();
			}
			
			_useMag = false;
			Invalidate();
			break;
		}
		
		case USE_MAGNIFIER:
		{
			if (_eyedropWindow->Lock()){
				_eyedropMsngr->SendMessage(Z_GO_AWAY);
				_running=false;
				_eyedropWindow->Unlock();
			}
			
			_useMag = true;
			Invalidate();
			break;
		}
		
		case REMOVE_FROM_TRAY:
		{
			thread_id tid = spawn_thread(removeFromDeskbar, "RemoveFromDeskbar", B_NORMAL_PRIORITY, NULL);
			if (tid) resume_thread(tid);
		
			break;
		}
		
		case OPEN_SETTINGS:
		{
			if (_settingsWindow->Lock())
			{
				
				if (_settingsWindow->IsHidden())
					_settingsWindow->Show();				

				if (!_settingsWindow->IsActive())
					_settingsWindow->Activate(true);
					
				if (_settingsWindow->IsMinimized())
					_settingsWindow->Minimize(false);
			
				_settingsWindow->Unlock();
			}
				
			break;
		}
		
		case MAGNIFY_WENT_TO_SLEEP: case EYEDROPPER_WENT_TO_SLEEP:
		{
			//magnify and eyedropper windows tell us when they are sent to sleep
			//so we can redraw the little '-' in the tray icon as '+'
			if (_running)
			{
				_running = false;
				Invalidate();
			}
			break;
		}
		
		default:
			BView::MessageReceived(message);
	}
}