#ifndef TRAY_VIEW
#define TRAY_VIEW

#define TRAY_SIG "magnify:traysig"
#define USE_EYEDROPPER 'zUED'
#define USE_MAGNIFIER 'zUMF'
#define REMOVE_FROM_TRAY 'zRFT'
#define OPEN_SETTINGS 'zOPS'

#include <TranslationKit.h>
#include <Deskbar.h>
#include <OS.h>


#include "common.h"
#include "magnifier.h"
#include "eyedropper.h"
#include "settings.h"


//exported instantiator function
extern "C" _EXPORT BView* instantiate_deskbar_item();


//since we can't remove the view from the deskbar from within the same thread
//as tray view, a thread will be spawned and this function called. It removed TrayView
//from the Deskbar
long removeFromDeskbar(void *);

class _EXPORT TrayView;


/*********************************************
	class TrayView derived from BView

	The icon in the Deskbar tray, provides the fundamental
	user interface. Archivable, so it can be flattened and fired
	at the deskbar.

*********************************************/

class TrayView:public BView{
	private:
		SettingsWindow *_settingsWindow;
		BMessenger *_magMsngr, *_eyedropMsngr;
		MagnifyWindow  *_magWindow;
		SwatchWindow *_eyedropWindow;
		bool _useMag, _running;
		
		BBitmap *_eyedropIcon, *_magIcon;
		BPath _appPath;
						
		void _init(void); //initialization common to all constructors 
		
	public:
		TrayView();
		TrayView(BMessage *mdArchive);
		virtual ~TrayView();
		
		virtual status_t Archive(BMessage *data, bool deep = true) const;
		static TrayView *Instantiate(BMessage *data);
		
		virtual void Draw(BRect updateRect );
		virtual void AttachedToWindow();
		virtual void MouseDown(BPoint where);
		virtual void MessageReceived(BMessage* message);
		virtual void GetPreferredSize(float *w, float *h);
};


/*********************************************
	ConfigMenu derived from BPopUpMenu
	Provides the contextual left-click menu for the
	TrayView. Fires it's messages at the TrayView specified
	in it's constructor;
	Also, it's by default set to asynchronously destruct,
	so it's basically a fire & forget kinda fella.
*********************************************/

class ConfigMenu: public BPopUpMenu{
	private:
	
	public:
		ConfigMenu(TrayView *tv, bool useMag);
		virtual ~ConfigMenu();
};


#endif