#include "settings.h"
#include <iostream.h>

DeskscopeSettings::DeskscopeSettings()
{
	//set to defaults, in case settings file doens't yet exist (eg, first time app is run)
	_eyedropperHex = false;
	_eyedropperClickToQuit = true; 
	_magnifyClickToQuit = false;
	_magnifyWindowSize = BRect(0, 0, 30, 30);
	_magnifyMagLevel = 3;


	BPath prefPath;

	find_directory(B_USER_SETTINGS_DIRECTORY, &prefPath);
	prefPath.Append(SETTINGS_FILE);
		
	_settingsFile.SetTo(prefPath.Path(), B_READ_WRITE | B_CREATE_FILE);
	
	if (_settingsMessage.Unflatten(&_settingsFile) == B_OK){
	
		if (_settingsMessage.FindBool(EYEDROPPER_COPY_MODE, &_eyedropperHex) != B_OK)
			printf("DeskscopeSettings::DeskscopeSettings();\tFailed to load eyedropper copy mode from settings file. Using default\n");
			
		if (_settingsMessage.FindBool(EYEDROPPER_CLICK_TO_QUIT, &_eyedropperClickToQuit) != B_OK)
			printf("DeskscopeSettings::DeskscopeSettings();\tFailed to load eyedropper exit mode from settings file. Using default\n");
			
		if (_settingsMessage.FindBool(MAGNIFY_CLICK_TO_QUIT, &_magnifyClickToQuit) != B_OK)
			printf("DeskscopeSettings::DeskscopeSettings();\tFailed to load magnify exit mode from settings file. Using default\n");
			
		if (_settingsMessage.FindRect(MAGNIFY_WINDOW_SIZE, &_magnifyWindowSize) != B_OK)
			printf("DeskscopeSettings::DeskscopeSettings();\tFailed to load window size from settings file. Using default\n");
			
		if (_settingsMessage.FindDouble(MAGNIFY_MAG_LEVEL, &_magnifyMagLevel) != B_OK)
			printf("DeskscopeSettings::DeskscopeSettings();\tFailed to load magnification level from settings file. Using default\n");

		if (_settingsMessage.FindFlat(DESKSCOPE_APP_PATH, (BFlattenable *) &_appPath) != B_OK)
			printf("DeskscopeSettings::DeskscopeSettings();\tFailed to load application path.\n");
	}
	else
	{
		printf("DeskscopeSettings::DeskscopeSettings()\nUnable to open settings file (either corrupted or doesn't exist), using defaults.\n");		
	}
	
	_settingsFile.Unset();		
}

DeskscopeSettings::~DeskscopeSettings()
{
	BPath prefPath;
	find_directory(B_USER_SETTINGS_DIRECTORY, &prefPath);
	prefPath.Append(SETTINGS_FILE);
	
	//clobber existing settings and write in new ones
	_settingsFile.SetTo(prefPath.Path(), B_READ_WRITE | B_ERASE_FILE);


	//empty message and refill it with whatever has been set
	_settingsMessage.MakeEmpty();

	_settingsMessage.AddBool(EYEDROPPER_COPY_MODE, _eyedropperHex);
	_settingsMessage.AddBool(EYEDROPPER_CLICK_TO_QUIT, _eyedropperClickToQuit);
	_settingsMessage.AddBool(MAGNIFY_CLICK_TO_QUIT, _magnifyClickToQuit);
	_settingsMessage.AddRect(MAGNIFY_WINDOW_SIZE, _magnifyWindowSize);
	_settingsMessage.AddDouble(MAGNIFY_MAG_LEVEL, _magnifyMagLevel);
	_settingsMessage.AddFlat(DESKSCOPE_APP_PATH, &_appPath);

	//write message to settings file
	if (_settingsMessage.Flatten(&_settingsFile) != B_OK)
		printf("Error occurred writing settings\n");

	_settingsFile.Unset();	
}

void DeskscopeSettings::SetEyedropperUseHex(bool yesno)
{
	_eyedropperHex = yesno;
}

bool DeskscopeSettings::EyedropperUseHex()
{
	return _eyedropperHex;
}

void DeskscopeSettings::SetEyedropperClickToQuit(bool yesno)
{
	_eyedropperClickToQuit = yesno;
}

bool DeskscopeSettings::EyedropperClickToQuit()
{
	return _eyedropperClickToQuit;
}

void DeskscopeSettings::SetMagnifyClickToQuit(bool yesno)
{
	_magnifyClickToQuit = yesno;
}

bool DeskscopeSettings::MagnifyClickToQuit()
{
	return _magnifyClickToQuit;
}

void DeskscopeSettings::SetMagnifyLevel(double l)
{
	_magnifyMagLevel = l;
}

double DeskscopeSettings::MagnifyLevel()
{
	return _magnifyMagLevel;
}

void DeskscopeSettings::SetMagnifyWindowSize(BRect s)
{
	_magnifyWindowSize = s;
}

BRect DeskscopeSettings::MagnifyWindowSize()
{
	return _magnifyWindowSize;
}

void DeskscopeSettings::SetAppPath(BPath p)
{
	_appPath = p;
}

BPath DeskscopeSettings::AppPath()
{
	return _appPath;
}
