#ifndef SETTINGS_H
#define SETTINGS_H


/***************************************************
	settings.h
	Mechanisms for managing the settings setting/retireval for Deskscope

	2000 Shamyl Zakariya
***************************************************/

//Include ZLayout layout manager
#include "zlayout.h"

#include "common.h"
#include "eyedropper.h"
#include "magnifier.h"

//used by settings window only
#define E_CLICK_TO_QUIT 'Zecq'
#define E_HEX_MODE 'Zhxm'
#define E_RGB_MODE 'Zrgm'
#define M_WINDOW_SIZE 'Zwdw'
#define M_CLICK_TO_QUIT 'Zctq'
#define M_MAG_LEVEL 'Zmgl'

/****************************************
	DeskscopeSettings
	Simple class for getting and setting prefs.
	Instantiating will open up settings file
	Destroying will write settings plus any changes
	back into the file.

	Settings file won't be updated until DeskscopeSettings
	destructor is called. Doens't matter if it's allocated off
	heap or stack. I recommend stack, though, to keep likelyhood
	of race conditions down.
	
	File is defined in common.h as SETTINGS_FILE

****************************************/
class DeskscopeSettings
{
	protected:
		BFile _settingsFile;
		BMessage _settingsMessage;
		
		bool _eyedropperHex;
		bool _eyedropperClickToQuit;
		bool _magnifyClickToQuit;
		BRect _magnifyWindowSize;
		double _magnifyMagLevel;
		BPath _appPath;

			
		BMessage openSettingsFile();
		void closeSettingsFile();
	
	public:
		DeskscopeSettings();
		~DeskscopeSettings();
		
		void SetEyedropperUseHex(bool yesno);
		bool EyedropperUseHex();
		
		void SetEyedropperClickToQuit(bool yesno);
		bool EyedropperClickToQuit();
		
		void SetMagnifyClickToQuit(bool yesno);
		bool MagnifyClickToQuit();
		
		void SetMagnifyLevel(double l);
		double MagnifyLevel();
		
		void SetMagnifyWindowSize(BRect s);
		BRect MagnifyWindowSize();
		
		void SetAppPath(BPath p);
		BPath AppPath();
};

/****************************************
	SettingsWindow derived form BWindow
	App modal window which provides way to
	set preferences for Deskscope. If an eyedropper
	or Magnifier are open which settings are changed, you'll
	see them happen. As changes are made internal prefs are 
	updated but changes are also broadcast to eyedropper
	and magnifier message loops. Very real-time

	Initializes via DeskscopeSettings class on construction and
	writes settings to disk when hidden or closed.
****************************************/

class SettingsWindow: public BWindow{
	protected:
		bool _eyedropperHex;
		bool _eyedropperClickToQuit;
		
		double _magnifyMagLevel;
		BRect _magnifyWindowSize;
		bool _magnifyClickToQuit;
		BPath _appPath;
				
		//these messengers send live configuration changes to eyedropper and magnifier
		//even though one or both may be hidden, their message loops will be running & listening
		BMessenger *_swMessenger, *_mgMessenger;

		void AssembleGui();
		void InvokeSettings();
		

	public:
		SettingsWindow(BMessenger *swMessenger, BMessenger *mgMessenger);
		virtual ~SettingsWindow();
		virtual bool QuitRequested();
		virtual void MessageReceived(BMessage* message);	
};


#endif