#include "settings.h"

SettingsWindow::SettingsWindow(BMessenger *swMessenger, BMessenger *mgMessenger)
	: BWindow( BRect( 50, 50, 370, 430 ),
	"Settings",
	B_MODAL_WINDOW_LOOK,
	B_MODAL_APP_WINDOW_FEEL,
	B_NOT_RESIZABLE | B_ASYNCHRONOUS_CONTROLS| B_NOT_ANCHORED_ON_ACTIVATE | B_NO_WORKSPACE_ACTIVATION)
{
	DeskscopeSettings settings;

	_eyedropperHex = settings.EyedropperUseHex();
	_eyedropperClickToQuit = settings.EyedropperClickToQuit();
	_magnifyMagLevel = settings.MagnifyLevel();
	_magnifyWindowSize = settings.MagnifyWindowSize();
	_magnifyClickToQuit = settings.MagnifyClickToQuit();

	_swMessenger = new BMessenger(*swMessenger);
	_mgMessenger = new BMessenger(*mgMessenger);	

		
	AssembleGui();
	InvokeSettings();		
	
	BScreen screen;
	float x = (screen.Frame().Width() / 2) - (Frame().Width() / 2);
	float y = (screen.Frame().Height() / 2) - (Frame().Height() / 2);
	
	MoveTo(x, y);
}

SettingsWindow::~SettingsWindow()
{
	delete _swMessenger;
	delete _mgMessenger;
}

bool SettingsWindow::QuitRequested()
{
	DeskscopeSettings settings;
	settings.SetEyedropperUseHex(_eyedropperHex);
	settings.SetEyedropperClickToQuit(_eyedropperClickToQuit);
	settings.SetMagnifyClickToQuit(_magnifyClickToQuit);
	settings.SetMagnifyLevel(_magnifyMagLevel);
	settings.SetMagnifyWindowSize(_magnifyWindowSize);

	
	Hide();
	return false; //so window isn't destroyed
}

void SettingsWindow::MessageReceived(BMessage* message)
{
	switch(message->what)
	{
		case E_RGB_MODE:
		{
			_eyedropperHex = false;
			
			BMessage msg(Z_SET_COPY_MODE);
			msg.AddBool(EYEDROPPER_COPY_MODE, _eyedropperHex);
			_swMessenger->SendMessage(&msg);
			break;
		}

		case E_HEX_MODE:
		{
			_eyedropperHex = true;
			
			BMessage msg(Z_SET_COPY_MODE);
			msg.AddBool(EYEDROPPER_COPY_MODE, _eyedropperHex);
			_swMessenger->SendMessage(&msg);

			break;
		}
		
		case E_CLICK_TO_QUIT:
		{
			int32 val = 0;
			message->FindInt32("be:value", &val);
			if (val > 0) 
			{
				_eyedropperClickToQuit = true; 
			}
			else
			{
				_eyedropperClickToQuit = false;
			}
			
			((BRadioButton *)FindView("eyedropperHexMode"))->SetEnabled(_eyedropperClickToQuit);
			((BRadioButton *)FindView("eyedropperRgbMode"))->SetEnabled(_eyedropperClickToQuit);

			
			
			BMessage msg(Z_SET_CLICK_TO_QUIT);
			msg.AddBool(EYEDROPPER_CLICK_TO_QUIT, _eyedropperClickToQuit);
			_swMessenger->SendMessage(&msg);

			break;			
		}
		
		case M_WINDOW_SIZE:
		{
			int32 val = 0;
			message->FindInt32("be:value", &val);
			if (val > 0)
			{
				_magnifyWindowSize.left = 0;
				_magnifyWindowSize.right = val;
				_magnifyWindowSize.top = 0;
				_magnifyWindowSize.bottom = val;
			}
			
			BMessage msg(Z_SET_WINDOW_SIZE);
			msg.AddRect(MAGNIFY_WINDOW_SIZE, _magnifyWindowSize);
			_mgMessenger->SendMessage(&msg);
			
			break;
		}
		
		case M_CLICK_TO_QUIT:
		{
			int32 val = 0;
			message->FindInt32("be:value", &val);
			if (val > 0) _magnifyClickToQuit = true; 
			else _magnifyClickToQuit = false;
			
			
			BMessage msg(Z_SET_CLICK_TO_QUIT);
			msg.AddBool(MAGNIFY_CLICK_TO_QUIT, _magnifyClickToQuit);
			_mgMessenger->SendMessage(&msg);

			break;
		}
		
		case M_MAG_LEVEL:
		{
			int32 val = 0;
			message->FindInt32("be:value", &val);
			if (val > 0)
			{
				_magnifyMagLevel = (double) val;
			}
			
			BMessage msg(Z_SET_MAGNIFICATION);
			msg.AddDouble(MAGNIFY_MAG_LEVEL, _magnifyMagLevel);
			_mgMessenger->SendMessage(&msg);
		}	
			
		default:
		{
			BWindow::MessageReceived(message);
			break;
		}
	}
}


void SettingsWindow::InvokeSettings()
{
	BMessage msg(Z_SET_COPY_MODE);
	msg.AddBool(EYEDROPPER_COPY_MODE, _eyedropperHex);
	_swMessenger->SendMessage(&msg);

	BMessage msg2(Z_SET_CLICK_TO_QUIT);
	msg2.AddBool(EYEDROPPER_CLICK_TO_QUIT, _eyedropperClickToQuit);
	_swMessenger->SendMessage(&msg2);

	BMessage msg3(Z_SET_WINDOW_SIZE);
	msg3.AddRect(MAGNIFY_WINDOW_SIZE, _magnifyWindowSize);
	_mgMessenger->SendMessage(&msg3);

	BMessage msg4(Z_SET_CLICK_TO_QUIT);
	msg4.AddBool(MAGNIFY_CLICK_TO_QUIT, _magnifyClickToQuit);
	_mgMessenger->SendMessage(&msg4);

	BMessage msg5(Z_SET_MAGNIFICATION);
	msg5.AddDouble(MAGNIFY_MAG_LEVEL, _magnifyMagLevel);
	_mgMessenger->SendMessage(&msg5);
	
	((BRadioButton *)FindView("eyedropperHexMode"))->SetEnabled(_eyedropperClickToQuit);
	((BRadioButton *)FindView("eyedropperRgbMode"))->SetEnabled(_eyedropperClickToQuit);	
}

void SettingsWindow::AssembleGui()
{
	ZLayoutView *top = new ZLayoutView(Bounds(), "top", new ZBorderLayout(), B_FOLLOW_ALL_SIDES);
	AddChild(top);

	ZLayoutView *center = new ZLayoutView(Z_NULLRECT, "center", new ZVerticalLayout());
	

	BStringView *eyedropperSV = new BStringView(Z_NULLRECT, "eyedropperSV", "Eyedropper");
	eyedropperSV->SetFont(be_bold_font);
	eyedropperSV->SetFontSize(14);	

	BStringView *magnifySV = new BStringView(Z_NULLRECT, "magnifySV", "Magnifier");
	magnifySV->SetFont(be_bold_font);
	magnifySV->SetFontSize(14);
	

	//SET UP THE EYEDROPPER PANEL AND INIT RADIO BUTTON TO VALUE IN SETTINGS FILE
	ZLayoutView *eyedropperSettings = new ZLayoutView(Z_NULLRECT, 
							"eyedropperSettings",
							new ZVerticalLayout(10), 
							B_FOLLOW_ALL_SIDES, B_FULL_UPDATE_ON_RESIZE);
	center->AddChild(eyedropperSV);
	center->AddChild(eyedropperSettings);
	
	BCheckBox *eyedropperQuitOnClick = new BCheckBox(Z_NULLRECT, "eyedropperQuitOnClick", "Quit Eyedropper on click (loss of focus)?", new BMessage(E_CLICK_TO_QUIT));
	eyedropperQuitOnClick->SetValue(_eyedropperClickToQuit);
		
	BStringView *eyedropperString = new BStringView(Z_NULLRECT, "eyedropperText", "Copy color to clipboard as hex or rgb?");

	BRadioButton *eyedropperHexMode = new BRadioButton(Z_NULLRECT, "eyedropperHexMode", "Hex", new BMessage(E_HEX_MODE));
	
	BRadioButton *eyedropperRgbMode = new BRadioButton(Z_NULLRECT, "eyedropperRgbMode", "RGB", new BMessage(E_RGB_MODE));
	_eyedropperHex ? eyedropperHexMode->SetValue(1) : eyedropperRgbMode->SetValue(1);
	
	eyedropperSettings->AddChild(eyedropperQuitOnClick);
	eyedropperSettings->AddChild(eyedropperString);
	eyedropperSettings->AddChild(eyedropperRgbMode);
	eyedropperSettings->AddChild(eyedropperHexMode);


	//SET UP THE MAGNIFICATION PANEL AND SET VALUES TO THOSE IN SETTINGS FILE
	ZLayoutView *magnifySettings = new ZLayoutView(Z_NULLRECT, 
							"magnififerSettings", 
							new ZVerticalLayout(10), 
							B_FOLLOW_ALL_SIDES, B_FULL_UPDATE_ON_RESIZE);
	center->AddChild(new ZSeparator(Z_NULLRECT, "magnify_sep", ZSeparator::HORIZONTAL));
	center->AddChild(magnifySV);
	center->AddChild(magnifySettings);

	BCheckBox *magnifyQuitOnClick = new BCheckBox(Z_NULLRECT, "quitOnClick", "Quit Magnifier on click (loss of focus)?", new BMessage(M_CLICK_TO_QUIT));
	magnifyQuitOnClick->SetValue(_magnifyClickToQuit);
	
	BSlider *magnificationSlider = new BSlider (Z_NULLRECT, "magnificationSlider", "Magnification:", new BMessage(M_MAG_LEVEL), 2.0, 10.0, B_TRIANGLE_THUMB);
	magnificationSlider->SetLimitLabels("2.0", "10.0");
	magnificationSlider->SetModificationMessage(new BMessage(M_MAG_LEVEL));
	magnificationSlider->SetPosition(_magnifyMagLevel / 8);
	magnificationSlider->SetHashMarks(B_HASH_MARKS_BOTTOM);
	magnificationSlider->SetHashMarkCount(9);
	magnificationSlider->ResizeToPreferred();

	BSlider *windowSizeSlider = new BSlider (Z_NULLRECT, "windowSizeSlider", "Window size (square):", new BMessage(M_WINDOW_SIZE), 20.0, 200.0, B_TRIANGLE_THUMB);
	windowSizeSlider->SetLimitLabels("20", "200");
	windowSizeSlider->SetModificationMessage(new BMessage(M_WINDOW_SIZE));
	windowSizeSlider->SetPosition(_magnifyWindowSize.Height() / 180);
	windowSizeSlider->SetHashMarks(B_HASH_MARKS_BOTTOM);
	windowSizeSlider->SetHashMarkCount(45);
	
	magnifySettings->AddChild(magnifyQuitOnClick);
	magnifySettings->AddChild(magnificationSlider);
	magnifySettings->AddChild(windowSizeSlider);
	

	//create south side
	ZLayoutView *south = new ZLayoutView(Z_NULLRECT, "south", new ZVerticalLayout());
	ZLayoutView *southTop = new ZLayoutView(Z_NULLRECT, "south", new ZHorizontalLayout());


	BButton *OKButton = new BButton(Z_NULLRECT, "ok", "Okay", new BMessage(B_QUIT_REQUESTED));
	OKButton->MakeDefault(true);
	
	BStringView *deskscopeSV = new BStringView(Z_NULLRECT, "deskscopeSV", "Deskscope");
	deskscopeSV->SetFont(be_bold_font);
	deskscopeSV->SetFontSize(24);
	
	southTop->AddChild(deskscopeSV);
	southTop->AddChild(OKButton);
	((ZHorizontalLayout *)southTop->LayoutManager())->PutSpring(1);
	
	south->AddChild(new ZSeparator(Z_NULLRECT, "deskscope.separator", ZSeparator::HORIZONTAL));
	south->AddChild(southTop);


	//we're done -- pack layout, and darken eyedropper and magnifier string views	
	eyedropperSV->SetViewColor(tint_color(top->ViewColor(), 1.07));
	eyedropperSV->SetLowColor(eyedropperSV->ViewColor());
	
	magnifySV->SetViewColor(tint_color(top->ViewColor(), 1.07));
	magnifySV->SetLowColor(magnifySV->ViewColor());



	top->AddChildAt(center, ZBorderLayout::CENTER);
	top->AddChildAt(south, ZBorderLayout::SOUTH);
	top->ResizeToPreferred();	
}