#include "frontend.h"

#define T_FOLLOW_FIRST 'ffst'
#define T_FOLLOW_SECOND 'fscn'
#define T_SPLIT_H 'spth'
#define T_SPLIT_V 'sptv'

ZLayoutView *top;

LayoutApp::LayoutApp()
	: BApplication( "application/x-vnd.zakariya.layout" )
{
	_window = new StubWindow;
	_window->Show();
		
	return;
}

LayoutApp::~LayoutApp()
{
	return;
}

void LayoutApp::AboutRequested(){

}

void LayoutApp::MessageReceived(BMessage* message)
{
	switch(message->what)
	{

		default:{
			BApplication::MessageReceived(message);
		}
	}
}

//*****************************************************************

TestView::TestView(BRect frame, const char* name, uint32 follow = B_FOLLOW_NONE, uint32 flags = B_WILL_DRAW | B_FULL_UPDATE_ON_RESIZE)
	: BView(frame, name, follow, flags)
{
	_prefWidth = frame.Width();
	_prefHeight = frame.Height();
}

TestView::~TestView() {}

void TestView::AttachedToWindow()
{
	SetViewColor(ui_color(B_PANEL_BACKGROUND_COLOR));
}
void TestView::Draw(BRect update)
{
	rgb_color lighter = tint_color(Parent()->ViewColor(), B_LIGHTEN_2_TINT);
	rgb_color darker = tint_color(Parent()->ViewColor(), B_DARKEN_3_TINT);
	BRect b(Bounds());

	SetPenSize(0);
	SetDrawingMode(B_OP_OVER);
	SetHighColor(ViewColor());
	SetLowColor(ViewColor());
	FillRect(b);
	
	SetHighColor(lighter);
	StrokeLine(b.LeftTop(), b.LeftBottom());
	StrokeLine(b.LeftTop(), b.RightTop());

	//draw diagonals.
	StrokeLine(b.LeftTop(), b.RightBottom());
	StrokeLine(b.RightTop(), b.LeftBottom());
	
	SetHighColor(darker);
	StrokeLine(b.RightTop(), b.RightBottom());
	StrokeLine(b.LeftBottom(), b.RightBottom());
	
	char str[64];
	sprintf(str, "(%d, %d)/(%d, %d)", (int)b.Width(), (int)b.Height(), (int)_prefWidth, (int)_prefHeight);
	BPoint center;

	SetDrawingMode(B_OP_ALPHA);
	SetBlendingMode(B_CONSTANT_ALPHA, B_ALPHA_OVERLAY);
	
	SetFontSize(64);
	font_height fh;
	GetFontHeight(&fh);
	float prefArea = _prefWidth * _prefHeight;
	float area = b.Width() * b.Height();
	float fw;

	SetHighColor(128, 155, 128, 64);	
	if (prefArea < area)
	{
		fw = StringWidth(">");
		center.x = b.Width() / 2 - fw / 2;
		center.y = b.Height() / 2 + fh.ascent / 4;				
		DrawString(">", center);
	}
	else if (prefArea > area)
	{
		fw = StringWidth("<");
		center.x = b.Width() / 2 - fw / 2;
		center.y = b.Height() / 2 - fh.ascent / 4;
		DrawString("<", center);
	}
	else
	{
		fw = StringWidth("=");
		center.x = b.Width() / 2 - fw / 2;
		center.y = b.Height() / 2 + fh.ascent / 4;
		DrawString("=", center);
	}
	
	SetHighColor(0,0,0, 180);
	SetFontSize(8);
	GetFontHeight(&fh);
	fw = StringWidth(str);
	
	center.x = b.Width() / 2 - fw / 2;
	center.y = b.Height() / 2 + fh.ascent / 2;
	DrawString(str, center);
}

void TestView::SetPreferredSize(float w, float h)
{
	_prefWidth = w;
	_prefHeight = h;
}

void TestView::GetPreferredSize(float *w, float *h)
{
	*w = _prefWidth;
	*h = _prefHeight;
}


//*****************************************************************


StubWindow::StubWindow()
	: BWindow( BRect( 100, 100, 500, 200 ), "Layout test", B_TITLED_WINDOW, 0)
{

	top = new ZLayoutView(Bounds(), "BorderView", new ZBorderLayout());

	BMenuBar *mb = new BMenuBar(Bounds(), "menu");
	BMenu *menu = new BMenu("File");
	menu->AddItem(new BMenuItem("New", new BMessage('stub'), 'N'));
	mb->AddItem(menu);
	top->AddChildAt(mb, ZBorderLayout::NORTH);


	ZSplitPane *centerSplit = new ZSplitPane(Z_NULLRECT, "splitter", ZSplitPane::VERTICAL);
	centerSplit->AddFirstChild(new TestView(BRect(0, 0, 300, 300), "first"));
	centerSplit->AddSecondChild(new TestView(BRect(0, 0, 300, 300), "second"));
	centerSplit->SetFollowingMode(ZSplitPane::FOLLOW_SECOND);
	
	top->AddChildAt(centerSplit, ZBorderLayout::CENTER);

	
	ZHorizontalLayout *hl = new ZHorizontalLayout();
	ZLayoutView *horizontalLayout = new ZLayoutView(Z_NULLRECT, "hl", hl);
	horizontalLayout->AddChild(new TestView(BRect(0, 0, 75, 75), "hltv0"));
	horizontalLayout->AddChild(new ZSeparator(Z_NULLRECT, "sep0", ZSeparator::VERTICAL));
	horizontalLayout->AddChild(new TestView(BRect(0, 0, 75, 65), "hltv2"));
	horizontalLayout->AddChild(new TestView(BRect(0, 0, 75, 55), "hltv3"));
	hl->PutSpring(3);
//	hl->SetFixedHeight(22);
//	hl->SetFixedWidth(100);
	
	top->AddChildAt(horizontalLayout, ZBorderLayout::SOUTH);

	ZVerticalLayout *vl = new ZVerticalLayout();
	ZLayoutView *verticalLayout = new ZLayoutView(Z_NULLRECT, "vl", vl);
	verticalLayout->AddChild(new TestView(BRect(0, 0, 75, 75), "vltv0"));
	verticalLayout->AddChild(new ZSeparator(Z_NULLRECT, "sep1", ZSeparator::HORIZONTAL));
	verticalLayout->AddChild(new TestView(BRect(0, 0, 55, 95), "vltv2"));
	vl->PutSpring(1);
//	vl->SetFixedHeight(22);
//	vl->SetFixedWidth(100);
	
	top->AddChildAt(verticalLayout, ZBorderLayout::EAST);


	//top->SetPreferredSize(400, 200);
	AddChild(top);
	top->ResizeToPreferred();
	return;
}

StubWindow::~StubWindow()
{

	return;
}

bool StubWindow::QuitRequested()
{
	be_app->PostMessage( B_QUIT_REQUESTED );
	
	return true;
}

void StubWindow::MessageReceived(BMessage* message)
{

	switch(message->what)
	{
		case 'notn':
		{		
			void *src;
			message->FindPointer("source", &src);
			BView *b = (BView *) src;
			ZLayoutView *lv = (ZLayoutView *) b->Parent();
			
			lv->RemoveChild(b);
			//top->ResizeToPreferred();
			break;
		}

		case T_FOLLOW_FIRST:
		{
			ZSplitPane *sp = (ZSplitPane *) FindView("splitter");
			if (sp)
			{
				sp->SetFollowingMode(ZSplitPane::FOLLOW_FIRST);			
			}
		
			break;
		}

		case T_FOLLOW_SECOND:
		{
			ZSplitPane *sp = (ZSplitPane *) FindView("splitter");
			if (sp)
			{
				sp->SetFollowingMode(ZSplitPane::FOLLOW_SECOND);			
			}
		
			break;
		}
		
		case T_SPLIT_V:
		{
			ZSplitPane *sp = (ZSplitPane *) FindView("splitter");
			if (sp)
			{
				sp->SetOrientation(ZSplitPane::VERTICAL);			
			}
		
			break;		
		}

		case T_SPLIT_H:
		{
			ZSplitPane *sp = (ZSplitPane *) FindView("splitter");
			if (sp)
			{
				sp->SetOrientation(ZSplitPane::HORIZONTAL);			
			}
		
			break;		
		}

		default:
			BWindow::MessageReceived(message);
	}
}


//********************************************************************************

int main( int argc, char **argv )
{
		
	BApplication *thisApp = new LayoutApp;
	thisApp->Run();
	delete thisApp;
	
	
	return 0;
} 
	