#include "zlayout.h"

ZBorderLayout::ZBorderLayout(int margin = 0) : ZLayoutManager(5)
{
	_nMargin = _sMargin = _eMargin = _wMargin = (float) margin;

	_ne = _nw = _se = _sw = CENTER;

}

ZBorderLayout::ZBorderLayout(int nM, int sM, int eM, int wM) : ZLayoutManager(5)
{
	_nMargin = nM;
	_sMargin = sM;	
	_eMargin = eM;
	_wMargin = wM;
		
	_ne = _nw = _se = _sw = CENTER;

}

ZBorderLayout::~ZBorderLayout(){ ; }

void ZBorderLayout::SetMargins(int n = 0, int s = 0, int e = 0, int w = 0)
{
	_nMargin = n;
	_sMargin = s;
	_eMargin = e;
	_wMargin = w;
}

void ZBorderLayout::Margins(int *n, int *s, int *e, int *w)
{
	*n = (int)_nMargin;
	*s = (int)_sMargin;
	*e = (int)_eMargin;
	*w = (int)_wMargin;
}


void ZBorderLayout::AddChild(BView *v)
{
	#ifdef DEBUG_BORDER
	printf("ZBorderLayout()::AddChild()\n");
	#endif;
	
	ZLayoutManager::AddChild(v);
	_associateCorners(v);
}

void ZBorderLayout::AddChildAt(BView *v, int32 at = 0)
{
	#ifdef DEBUG_BORDER
	printf("ZBorderLayout()::AddChildAt\n");
	#endif;


	ZLayoutManager::AddChildAt(v, at);
	_associateCorners(v);
}

void ZBorderLayout::RemoveChild(BView *v)
{
	#ifdef DEBUG_BORDER
	printf("ZBorderLayout()::RemoveChild()\n");
	#endif;

	_disAssociateCorners(v);	
	ZLayoutManager::RemoveChild(v);
}

void ZBorderLayout::RemoveChild(int32 i)
{
	#ifdef DEBUG_BORDER
	printf("ZBorderLayout()::RemoveChild()\n");
	#endif;
	
	_disAssociateCorners(ChildAt(i));
	ZLayoutManager::RemoveChild(i);
}


void ZBorderLayout::_associateCorners(BView *v)
{
	#ifdef DEBUG_BORDER
	printf("ZBorderLayout()::_associateCorners()\n");
	#endif;


	switch(IndexOfChild(v))
	{
	
		case NORTH:
			{	
				if (_ne == CENTER) _ne = NORTH;
				if (_nw == CENTER) _nw = NORTH;
				break;
			}

			case EAST:
			{	
				if (_ne == CENTER) _ne = EAST;
				if (_se == CENTER) _se = EAST;	
				break;
			}

			case SOUTH:
			{	
				if (_se == CENTER) _se = SOUTH;
				if (_sw == CENTER) _sw = SOUTH;	
	
				break;
			}

			case WEST:
			{	
				if (_nw == CENTER) _nw = WEST;
				if (_sw == CENTER) _sw = WEST;	
	
				break;
			}

			case CENTER: default:
			{	
					
				break;
			}	
	}
}


void ZBorderLayout::_disAssociateCorners(BView *v)
{
	#ifdef DEBUG_BORDER
	printf("ZBorderLayout()::_disAssociateCorners()\n");
	#endif;


	switch(IndexOfChild(v))
	{
	
		case NORTH:
			{	
				if (_ne == NORTH) _ne = (ChildAt(EAST) ? EAST : CENTER);
				if (_nw == NORTH) _nw = (ChildAt(WEST) ? WEST : CENTER);
				break;
			}

			case EAST:
			{	
				if (_ne == EAST) _ne = (ChildAt(NORTH) ? NORTH : CENTER);
				if (_se == EAST) _se = (ChildAt(SOUTH) ? SOUTH : CENTER);
				break;
			}

			case SOUTH:
			{	
				if (_se == SOUTH) _se = (ChildAt(EAST) ? EAST : CENTER);
				if (_sw == SOUTH) _sw = (ChildAt(WEST) ? WEST : CENTER);
	
				break;
			}

			case WEST:
			{	
				if (_nw == WEST) _nw = (ChildAt(NORTH) ? NORTH : CENTER);
				if (_sw == WEST) _sw = (ChildAt(SOUTH) ? SOUTH : CENTER);
	
				break;
			}

			case CENTER: default:
			{	
					
				break;
			}	
	}
}


//******************************************************************************
void ZBorderLayout::Layout(float width, float height)
{
	#ifdef DEBUG_BORDER	
	printf("ZBorderLayout::Layout()\n");
	#endif
	
	BView *v = NULL;
	BRect f;
	float pw, ph; //preferred width, height
	float margins[4];
	
	//we need to initialize margins array to user settings or preferred sizes of contents
	for (int i = 0; i < 4; i++)
	{
		v = _viewList[i];
		if (v)
		{
			switch(i)
			{
				case NORTH:
				{
					if (_nMargin) margins[NORTH] = _nMargin;
					else 
					{
						_viewList[NORTH]->GetPreferredSize(&pw, &ph);
						margins[NORTH] = ph;
					}

					break;
				}

				case SOUTH:
				{
					if (_sMargin) margins[SOUTH] = _sMargin;
					else 
					{
						_viewList[SOUTH]->GetPreferredSize(&pw, &ph);
						margins[SOUTH] = ph;
					}

					break;
				}
				
				case EAST:
				{
					if (_eMargin) margins[EAST] = _eMargin;
					else 
					{
						_viewList[EAST]->GetPreferredSize(&pw, &ph);
						margins[EAST] = pw;
					}

					break;
				}
				
				case WEST:
				{
					if (_wMargin) margins[WEST] = _wMargin;
					else 
					{
						_viewList[WEST]->GetPreferredSize(&pw, &ph);
						margins[WEST] = pw;
					}

					break;
				}								
			}
		}
	}
	
	for (int i = 0; i < 5; i++){
		v = _viewList[i];		
		if (v){
			f.top = 0; f.left = 0; f.right = width; f.bottom = height;
				
			switch(i){
	
				case NORTH:
				{

					#ifdef DEBUG_BORDER	
					printf("\tNorth\n");
					#endif

					f.bottom = margins[NORTH] - 1;
					if (_nw != NORTH) f.left = margins[WEST];
					if (_ne != NORTH) f.right -= margins[EAST] + 1;
					break;
				}
	
				case EAST:
				{	
					#ifdef DEBUG_BORDER	
					printf("\tEast\n");
					#endif


					f.left = f.right - margins[EAST];
					if (_ne != EAST) f.top = margins[NORTH];
					if (_se != EAST) f.bottom -= margins[SOUTH] + 1;
					break;
				}
	
				case SOUTH:
				{	

					#ifdef DEBUG_BORDER	
					printf("\tSouth\n");
					#endif

					f.top = f.bottom - margins[SOUTH];
					if (_se != SOUTH) f.right -= margins[EAST] + 1;
					if (_sw != SOUTH) f.left = margins[WEST];
					break;
				}
	
				case WEST:
				{	

					#ifdef DEBUG_BORDER	
					printf("\tWest\n");
					#endif

					f.right = margins[WEST] - 1;				
					if (_nw != WEST) f.top = margins[NORTH];
					if (_sw != WEST) f.bottom -= margins[SOUTH] + 1;
		
					break;
				}
	
				case CENTER: default:
				{	

					#ifdef DEBUG_BORDER	
					printf("\tCenter\n");
					#endif

					if (ChildAt(NORTH))					
						f.top = margins[NORTH];
						
					if (ChildAt(WEST))					
						f.left = margins[WEST];
						
					if (ChildAt(EAST))
						f.right = width - (margins[EAST] + 1);
						
					if (ChildAt(SOUTH))
						f.bottom = height - (margins[SOUTH] + 1);
										
					break;
				}
			}
			v->MoveTo(f.left, f.top);
			v->ResizeTo(f.Width(), f.Height());		
			
			#ifdef DEBUG_BORDER	
			printf("\tResized to");
			f.PrintToStream();
			printf("\n\n");
			#endif

		}
	}
}

void ZBorderLayout::Draw(ZLayoutView *v, float w, float h)
{
	rgb_color c = tint_color(v->ViewColor(), B_LIGHTEN_1_TINT);
	
	v->SetPenSize(0);
	BRect bnds = BRect(0, 0, w, h);
	v->SetHighColor(c);
	v->StrokeRoundRect(bnds, 3, 3);
}

void ZBorderLayout::GetPreferredSize(float *width, float *height, float currentWidth, float currentHeight)
{
	float cW = 0, cH = 0;
	float n = 0, s = 0, e = 0, w = 0;
	float nWidth = 0, sWidth = 0, eHeight = 0, wHeight = 0, maxWidth = 0, maxHeight = 0;
	float pw, ph;
	
	if (ChildAt(EAST))
	{
		ChildAt(EAST)->GetPreferredSize(&pw, &ph);
		if (_eMargin) e = _eMargin;
		else e = pw;
		
		eHeight = ph;
	}

	if (ChildAt(WEST))
	{
		ChildAt(WEST)->GetPreferredSize(&pw, &ph);
		if (_wMargin) w = _wMargin;
		else w = pw;
		
		wHeight = ph;
	}
	
	if (ChildAt(NORTH)){
		ChildAt(NORTH)->GetPreferredSize(&pw, &ph);
		if (_nMargin) n = _nMargin;
		else n = ph;
		
		nWidth = pw;
	}
	
	if (ChildAt(SOUTH)){
		ChildAt(SOUTH)->GetPreferredSize(&pw, &ph);
		if (_sMargin) s = _sMargin;
		else s = ph;
		
		sWidth = pw;
	}
	
	if (ChildAt(CENTER))
		ChildAt(CENTER)->GetPreferredSize(&cW, &cH);

	maxWidth = (nWidth > sWidth) ? nWidth : sWidth;
	if (cW > maxWidth) maxWidth = cW;

	maxHeight = (eHeight > wHeight) ? eHeight : wHeight;
	if (cH > maxHeight) maxHeight = cH;
		
	*width = w + e + maxWidth;
	*height = n + s + maxHeight;
}