#include "zlayout.h"

/**
* ZLayoutManager -- Base class of layout managers which actually do something
* Responsibility:
* Handle addition and removal of views from list
*
*
**/

ZLayoutManager::ZLayoutManager(int32 size)
{
	_size = size;
	_viewList = new (BView *)[_size];
	
	for (int i =0; i < _size; i++)
		_viewList[i] = NULL;
		
	SetDrawing(false);
}

ZLayoutManager::~ZLayoutManager()
{
	#ifdef DEBUG_BASE
	printf("ZLayoutManager::~ZLayoutManager()\n");
	#endif
	
	if (_viewList) delete _viewList;
}

void ZLayoutManager::SetDrawing(bool tf)
{
	_doDraw = tf;
}

bool ZLayoutManager::ShouldDraw()
{
	return _doDraw;
}

void ZLayoutManager::AddChild(BView *v){
	//find first NULL position and put it there

	for (int i = 0; i < _size; i++)
		if (_viewList[i] == NULL){
			_viewList[i] = v;
			break;
		}
}


void ZLayoutManager::AddChildAt(BView *v, int32 where){
	if (CountChildren() >= Size()){
		#ifdef DEBUG_BASE
		printf("ZLayoutManager::AddChildAt(...) : No room for more views.");
		#endif
		return;
	}
	
	if ((where >= Size()) || (where < 0)){
		#ifdef DEBUG_BASE
		printf("ZLayoutManager::AddChildAt(...) : %ld not in range of container array.", where);
		#endif
		return;
	}
	if (!_viewList[where]) _viewList[where] = v;
}

void ZLayoutManager::AddChildWithConstraint(BView *v, ZConstraint *constraint){
	//Since it takes a special subclass to do 
	//anything useful with constraints we just call AddChild

	AddChild(v);
}

int32 ZLayoutManager::Size(){
	return _size;
}

int32 ZLayoutManager::CountChildren(){
	int32 count = 0;
	
	for (int i = 0; i < _size; i++)
		if (_viewList[i] != NULL) count++;
		
	return count;
}

void ZLayoutManager::RemoveChild(BView *v){
	_viewList[IndexOfChild(v)] = NULL;	
}

void ZLayoutManager::RemoveChild(int32 i){
	_viewList[i] = NULL;
}


BView * ZLayoutManager::ChildAt(int32 i){
	if ((i < 0) || (i > _size)) return NULL;
	
	return _viewList[i];
}

int32 ZLayoutManager::IndexOfChild(BView *v){
	for (int32 i = 0; i < _size; i++)
		if (_viewList[i] == v) return i;
		
	return -1;
}

void ZLayoutManager::GetPreferredSize(float *w, float *h, float currentWidth, float currentHeight)
{
	//default implementation
	//tells ZLayoutView to ignore this. IE, ZLayoutView will just 
	//use Bounds().Width() & Bounds().Height()
	*w = -1;
	*h = -1;
}

