#include "zlayout.h"

ZSeparator::ZSeparator(BRect frame, const char *name, long orientation, float padding = 0, uint32 follow = B_FOLLOW_NONE)
	: BView(frame, name, follow, B_WILL_DRAW)
{
	_padding = padding;
	_orient = orientation;
}

ZSeparator::~ZSeparator(){}

void ZSeparator::Draw(BRect update)
{
	BRect b(Bounds());

	SetPenSize(0);
	SetHighColor(ViewColor());
	FillRect(b);	

	rgb_color light = tint_color(ViewColor(), B_LIGHTEN_2_TINT);
	rgb_color dark = tint_color(ViewColor(), B_DARKEN_2_TINT);
	
	if (_orient == HORIZONTAL)
	{
		BPoint p(b.LeftTop()), q(b.RightTop());
		p.y = _padding + 1; q.y = _padding + 1;
		
		SetHighColor(dark);
		StrokeLine(p, q);		

		p.y++; q.y++;
		SetHighColor(light);
		StrokeLine(p, q);		
	}
	else
	{
		BPoint p(b.LeftTop()), q(b.LeftBottom());
		p.x = _padding + 1; q.x = _padding + 1;
		
		SetHighColor(dark);
		StrokeLine(p, q);		

		p.x++; q.x++;
		SetHighColor(light);
		StrokeLine(p, q);		
	}	
}

void ZSeparator::GetPreferredSize(float *w, float *h)
{
	if (_orient == HORIZONTAL)
	{
		*w = 0;
		*h = 2 * _padding + 2;
	}
	else
	{
		*h = 0;
		*w = 2 * _padding + 2;
	}	
}

void ZSeparator::AttachedToWindow()
{
	if (Parent()) SetViewColor(Parent()->ViewColor());
	else SetViewColor(ui_color(B_PANEL_BACKGROUND_COLOR));
}