#include "zlayout.h"

ZVerticalLayout::ZVerticalLayout(int hPad = 2, int vPad = 2, int children = 36)
	:ZLayoutManager(children)
{
	_spring = -1;
	_hPad = hPad;
	_vPad = vPad;

	_fixedWidth = 0;
	_fixedHeight = 0;
	
	_childRects = new BRect[children];
	SetJustified(true);
}

ZVerticalLayout::~ZVerticalLayout(){ delete [] _childRects;}

void ZVerticalLayout::PutSpring(int where)
{
	if (where > CountChildren()) where = CountChildren() - 1;
	if (where < 0) where = -1;

	_spring = where;
}

void ZVerticalLayout::RemoveSpring()
{
	_spring = -1;
}

void ZVerticalLayout::SetJustified(bool j)
{
	_justify = j;
}

bool ZVerticalLayout::Justified()
{
	return _justify;
}

void ZVerticalLayout::SetFixedWidth(float w = 0)
{
	_fixedWidth = w;
}

float ZVerticalLayout::FixedWidth()
{
	return _fixedWidth;
}

void ZVerticalLayout::SetFixedHeight(float h)
{
	_fixedHeight = h;
}

float ZVerticalLayout::FixedHeight()
{
	return _fixedHeight;
}

void ZVerticalLayout::Layout(float w, float h)
{
	//first, we lay out all the children into index correspondent _childRects
	//then we deternime where to put the spring (if at all) and reposition those after _spring
	//relative to the opposite edge
	//then we put the children into their corresponding positions (based on the rects)
	
	int size = Size();

	//calculate where children would go sans a spring
	int rectIndex = 0;
	for (int i = 0; i < size; i++)
	{
		BView *child = ChildAt(i);
		if (child)
		{
			//much to decide here
			//first determine how big child would like to be
			//if we've set a preferred width or height, ovveride GetPreferredSize
			//now, since layout is vertical, if layout is justified, we need to stretch each component to edge
			//but only if preferred widths aren't set.
			
			float prefWidth, prefHeight;
			child->GetPreferredSize(&prefWidth, &prefHeight);
			if (_fixedHeight) prefHeight = _fixedHeight;
						
			if (_fixedWidth) prefWidth = _fixedWidth;
			else if (_justify) prefWidth = w - 2 *_hPad;

			//special case for first child in row, position relative to upper left of view (with padding)							
			if (rectIndex == 0)
			{
				_childRects[rectIndex].Set(_hPad, _vPad, _hPad + prefWidth, _vPad + prefHeight);
			}
			else //position the n relative to (n - 1)
			{
				BRect *lastRect = &_childRects[rectIndex - 1];
				_childRects[rectIndex].Set(_hPad, _vPad + lastRect->bottom, _hPad + prefWidth, _vPad + prefHeight + lastRect->bottom);
			}
			
			rectIndex++;
		}
	}
	
	//if there's a spring, move _childRects at and after _spring to align to right-hand boundary
	if (_spring >= 0)
	{
		float dy = 0;

		//find .bottom of last child		
		for (int i = _spring; i < size; i++)
		{
			if (ChildAt(i))
				dy = _childRects[i].bottom;
		}
		
		//how far must they be translated?
		dy = h - dy - _vPad;
		
		if (dy > 0)
		{
			for (int i = _spring; i < size; i++)
			{
				if (ChildAt(i)) _childRects[i].OffsetBy(0, dy);		
			}
		}
	}
	
	//position children per what we've calculated.
	for (int i = 0; i < size; i++)
	{
		BView *child = ChildAt(i);
		if (child)
		{
			//don't resize/move if it's already at the right size/position
			//prevents a little flickering
			if (_childRects[i] != child->Frame())
			{
				child->MoveTo(_childRects[i].LeftTop());
				child->ResizeTo(_childRects[i].Width(), _childRects[i].Height());
			}
		}	
	}
}

void ZVerticalLayout::GetPreferredSize(float *w, float *h, float currentWidth, float currentHeight)
{
	float maxChildWidth = 0, summedHeights = _vPad;
	float th, tw;
	int size = Size();

	for (int i = 0; i < size; i++)
	{
		BView *child = ChildAt(i);
		if (child)
		{	
			child->GetPreferredSize(&tw, &th);
			if (tw > maxChildWidth) maxChildWidth = tw;
			summedHeights += th + _vPad;
		}
	}

	maxChildWidth += 2*_hPad;

	//if user has chosen a preferred width or height, set it so
	if (_fixedHeight) summedHeights = (_fixedHeight + _vPad) * CountChildren() + _vPad;
	if (_fixedWidth) maxChildWidth = _fixedWidth + 2 * _hPad;

	*w = maxChildWidth;
	*h = summedHeights;
}